@@ -19,9 +19,9 @@
 #ifndef EMOTICON_FILTER_H
 #define EMOTICON_FILTER_H
 
-#include <KTp/AbstractMessageFilter>
+#include <KTp/abstract-message-filter.h>
 
-class EmoticonFilter : public AbstractMessageFilter
+class EmoticonFilter : public KTp::AbstractMessageFilter
 {
 Q_OBJECT
 
@@ -29,10 +29,10 @@ public:
     EmoticonFilter(QObject* parent, const QVariantList &);
     virtual ~EmoticonFilter();
 
-    void filterMessage(Message& message);
+    void filterMessage(KTp::Message &message, const KTp::MessageContext &context);
 private:
     class Private;
     Private *d;
 };
 
-#endif
\ No newline at end of file
+#endif
