@@ -26,6 +26,8 @@
 #include <QtCore/QTimer>
 #include <QtCore/QDebug>
 #include <QtCore/QString>
+#include <QApplication>
+#include <QClipboard>
 
 #include <KStandardShortcut>
 #include <KActionCollection>
@@ -76,8 +78,7 @@ void ChatTextEdit::setFontBold(bool isBold)
 QSize ChatTextEdit::minimumSizeHint() const
 {
     QSize sh = KTextEdit::minimumSizeHint();
-    sh.setHeight(fontMetrics().height() + 1);
-    sh += QSize(0, QFrame::lineWidth() * 2);
+    sh.setHeight(2 * fontMetrics().height() + fontMetrics().lineSpacing());
     return sh;
 }
 
@@ -176,6 +177,14 @@ void ChatTextEdit::updateScrollBar()
     ensureCursorVisible();
 }
 
+void ChatTextEdit::pasteSelection()
+{
+    const QMimeData *md = QApplication::clipboard()->mimeData(QClipboard::Selection);
+    if (md) {
+        insertFromMimeData(md);
+    }
+}
+
 void ChatTextEdit::sendMessage()
 {
     if (!toPlainText().isEmpty()) {
